from .tag_word import tag_word

def tokenize(headline: str) -> list[tuple[str, str]]:
  """Tokenize a headline into a list of tuples."""
  words = headline.split()

  tokens = [tag_word(word) for word in words]
  return tokens

def get_tag(token: tuple[str, str]):
  """Get the tag of a token."""
  return token[1]


from .wordbank import power_words
from .wordbank import uncommon_words
from .tokenize import tokenize
from .calculate_wordbank_percentage import calculate_wordbank_percentage

def test_calculte_wordbank_percentage():
    power_word = iter(power_words)
    uncommon_word = iter(uncommon_words)

    test_case = tokenize(f'1 {next(uncommon_word)} 3 { next(power_word) } 5 { next(power_word) } 7 8 9 10')

    result = calculate_wordbank_percentage(test_case)

    assert result['power'] == 0.2
    assert result['uncommon'] == 0.1




from typing import Literal
from .power_words import power_words
from .uncommon_words import uncommon_words

def tag_from_wordbanks(word: str) -> Literal['power' , 'uncommon', '']:
  word_lower = word.lower()
  if word_lower in power_words:
    return 'power'
  elif word_lower in uncommon_words:
    return 'uncommon'
  else:
    return ''


from random import random
from .tokenize import tokenize

def test_tokenize():
    test_case = 'this is a headline for testing'
    test_case_words = test_case.split()

    result = tokenize(test_case)

    assert type(result) == list
    assert type(result[0]) == tuple
    assert type(result[0][0]) == str
    assert result[0][0] == test_case_words[0]

    random_index = int(random() * len(test_case_words))
    assert result[random_index][0] == test_case_words[random_index]

    assert result[-1][0] == test_case_words[-1]



power_words = {
"bargain",
"before",
"best",
"big",
"bigger",
"billion",
"bonanza",
"bonus",
"cash",
"cheap",
"deadline",
"discount",
"dollar",
"don’t miss out",
"double",
"economical",
"exclusive",
"expires",
"explode",
"extra",
"fast",
"feast",
"final",
"first",
"fortune"
}


from .infer import infer_headline_stats

def test_infer_headline_stats():
  test_case_high = {'word_count': 18, 'headline': 'test headline 1'}
  test_case_low = {'word_count': 2, 'headline': 'test headline 2'}
  test_case_optimal = {'word_count': 9, 'headline': 'test headline 3'}

  # assertions
  result_high = infer_headline_stats(test_case_high)
  assert result_high['word_count_inference'] == 'high'

  result_low = infer_headline_stats(test_case_low)
  assert result_low['word_count_inference'] == 'low'
  assert result_low['word_count_message'] == 'word count is too low'

  result_optimal = infer_headline_stats(test_case_optimal)
  assert result_optimal['word_count_inference'] == 'optimal'


from .count_words import count_words
from .count_characters import count_characters

def get_headline_stats(headline: str) -> dict:
  if(type(headline) != str):
    raise TypeError('headline must be a string')

  result = {
    "word_count": count_words(headline), 
    "character_count": count_characters(headline)
  }
  return result



from .count_characters import count_characters

def test_count_characters():
    test_10 = 'x' * 10
    test_1001 = 'x' * 1001
    test_999 = 'x' * 999
    
    assert count_characters(test_10) == 10
    assert count_characters(test_1001)== 1001
    assert count_characters(test_999)== 999


from functools import reduce
from .tokenize import get_tag
from itertools import count


def calculate_wordbank_percentage(tokens: list[tuple[str, str]]):
	"""Calculate the percentage of each word in a list of tokens."""
	length = len(tokens)

	# time complexity (O(len(tokens)))

	countDict: dict[str, int] = {}
	percentDict: dict[str, float] = {}

	for token in tokens:
		tag = get_tag(token)
		if(not(tag in countDict)):
			countDict[tag] = 1
		else:
			countDict[tag] += 1
		percentDict[tag] = countDict[tag]/length

	return percentDict






from .stats import get_headline_stats
from .inference import infer_headline_stats
from .wordbank import tokenize
from .wordbank import calculate_wordbank_percentage

def analyze_headline(headline: str):
  """Analyze a headline."""

  tokens = tokenize(headline)
  stats = get_headline_stats(headline)
  inference = infer_headline_stats(stats)
  percentage = calculate_wordbank_percentage(tokens)

  result = {
    'headline': headline,
    'tokens': tokens,
    'stats': stats,
    'inference': inference,
    'percentage': percentage
  }

  return result


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['headline_analyzer',
 'headline_analyzer.goodheadlines',
 'headline_analyzer.inference',
 'headline_analyzer.stats',
 'headline_analyzer.utils',
 'headline_analyzer.wordbank']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'headline-analyzer',
    'version': '0.1.0',
    'description': 'Analyze headlines for optimising conversion rate',
    'long_description': None,
    'author': 'Nivrith',
    'author_email': 'nivrith.mandayam@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



def infer_headline_stats(headlines_stats: dict) -> dict:
  """Infer headline stats from a list of headlines stats."""
  # Infer word count
  word_count_message = 'word count is optimal'
  word_count_inference = 'optimal'

  if headlines_stats['word_count'] < 4:
    word_count_message = 'word count is too low'
    word_count_inference = 'low'
  elif headlines_stats['word_count'] > 15:
    word_count_message = 'word count is too high'
    word_count_inference = 'high'

  result = {
    'word_count_message': word_count_message,
    'word_count_inference': word_count_inference
  }

  return result


from .analyze_headline import analyze_headline

def test_analyze_headline():
  test_headlines = [
    'single',
    'this is a test headline',
    'test headline 2',
    'test headline 3 with a lot of words'
  ]
  # assert features
  for test_headline in test_headlines:
    test_result = analyze_headline(test_headline)
    assert test_result['headline'] == test_headline
    assert test_result['stats']['word_count'] == len(test_headline.split())
